@@ -27,10 +27,20 @@ hunspell dictonaries.
 from typing import Dict
 from typing import Tuple
 from typing import List
-from typing import Optional
 import os
 import sys
 import unicodedata
+IMPORT_REGEX_SUCCESFUL = False
+try:
+    import regex # type: ignore
+    IMPORT_REGEX_SUCCESFUL = True
+    # Enable new improved regex engine instead of backwards compatible
+    # v0.  regex.match('ß', 'SS', regex.IGNORECASE) matches only with
+    # the improved version!  See also: https://pypi.org/project/regex/
+    regex.DEFAULT_VERSION = regex.VERSION1
+except (ImportError,):
+    # Use standard “re” module as a fallback:
+    import re
 import logging
 import itb_util
 
@@ -65,95 +75,6 @@ except (ImportError,):
 # letter of a word until the candidate lookup table pops up.
 MAX_WORDS = 100
 
-# List of languages where accent insensitive matching makes sense:
-ACCENT_LANGUAGES = {
-    'af': '',
-    'ast': '',
-    'az': '',
-    'be': '',
-    'bg': '',
-    'br': '',
-    'bs': '',
-    'ca': '',
-    'cs': '',
-    'csb': '',
-    'cv': '',
-    'cy': '',
-    'da': 'æÆøØåÅ',
-    'de': '',
-    'dsb': '',
-    'el': '',
-    'en': '',
-    'es': '',
-    'eu': '',
-    'fi': 'åÅäÄöÖ',
-    'fo': '',
-    'fr': '',
-    'fur': '',
-    'fy': '',
-    'ga': '',
-    'gd': '',
-    'gl': '',
-    'grc': '',
-    'gv': '',
-    'haw': '',
-    'hr': '',
-    'hsb': '',
-    'ht': '',
-    'hu': '',
-    'ia': '',
-    'is': '',
-    'it': '',
-    'kk': '',
-    'ku': '',
-    'ky': '',
-    'lb': '',
-    'ln': '',
-    'lv': '',
-    'mg': '',
-    'mi': '',
-    'mk': '',
-    'mn': '',
-    'mos': '',
-    'mt': '',
-    'nb': 'æÆøØåÅ',
-    'nds': '',
-    'nl': '',
-    'nn': 'æÆøØåÅ',
-    'nr': '',
-    'nso': '',
-    'ny': '',
-    'oc': '',
-    'pl': '',
-    'plt': '',
-    'pt': '',
-    'qu': '',
-    'quh': '',
-    'ru': '',
-    'sc': '',
-    'se': '',
-    'sh': '',
-    'shs': '',
-    'sk': '',
-    'sl': '',
-    'smj': '',
-    'sq': '',
-    'sr': '',
-    'ss': '',
-    'st': '',
-    'sv': 'åÅäÄöÖ',
-    'tet': '',
-    'tk': '',
-    'tn': '',
-    'ts': '',
-    'uk': '',
-    'uz': '',
-    've': '',
-    'vi': '',
-    'wa': '',
-    'xh': '',
-}
-
 class Dictionary:
     '''A class to hold a hunspell dictionary
     '''
@@ -184,10 +105,10 @@ class Dictionary:
          self.encoding,
          self.words) = itb_util.get_hunspell_dictionary_wordlist(self.name)
         if self.words:
-            if self.language in ACCENT_LANGUAGES:
+            if self.language in itb_util.ACCENT_LANGUAGES:
                 self.word_pairs = [
                     (x, itb_util.remove_accents(
-                        x, keep=ACCENT_LANGUAGES[self.language]))
+                        x, keep=itb_util.ACCENT_LANGUAGES[self.language]))
                     for x in self.words
                 ]
             for word in self.words:
@@ -199,8 +120,12 @@ class Dictionary:
             if self.name.split('_')[0] == 'fi':
                 self.enchant_dict = None
                 self.pyhunspell_object = None
-                if IMPORT_LIBVOIKKO_SUCCESSFUL:
-                    self.voikko = libvoikko.Voikko('fi')
+                self.voikko = None
+                if not IMPORT_LIBVOIKKO_SUCCESSFUL:
+                    LOGGER.warning(
+                        'Language is “fi” but “import libvoikko” failed.')
+                    return
+                self.voikko = libvoikko.Voikko('fi')
                 return
             if IMPORT_ENCHANT_SUCCESSFUL:
                 try:
@@ -274,9 +199,7 @@ class Dictionary:
         Spellcheck a word using enchant, pyhunspell, or voikko
 
         :param word: The word to spellcheck
-        :type word: String
         :return: True if spelling is correct, False if not or unknown
-        :rtype: Boolean
 
         >>> d = Dictionary('en_US')
         >>> d.spellcheck('winter')
@@ -305,7 +228,6 @@ class Dictionary:
         Returns wether this dictionary supports spellchecking or not
 
         :return: True if this dictionary spports spellchecking, False if not
-        :rtype: Boolean
 
         Examples:
 
@@ -371,7 +293,7 @@ class Dictionary:
         :param word: The word to return spellchecking suggestions for
         :return: List of spellchecking suggestions, possibly empty.
         '''
-        if not word:
+        if not word or not self.voikko:
             return []
         return [
             unicodedata.normalize(
@@ -385,15 +307,18 @@ class Dictionary:
         Return spellchecking suggestions for word using enchant, pyhunspell or voikko
 
         :param word: The word to return spellchecking suggestions for
-        :type word: String
         :return: List of spellchecking suggestions, possibly empty.
-        :rtype: List of strings
+
+        Results can be quite different depending on whether enchant or pyhunspell
+        is used and in case of enchant whether hunspell, aspell,
+        myspell, or ispell are used. So for the doctests, better don’t
+        check the complete list of suggestions returned.
 
         Examples:
 
         >>> d = Dictionary('en_US')
-        >>> d.spellcheck_suggest('kamel')
-        ['camel', 'Camel']
+        >>> 'Camel' in d.spellcheck_suggest('kamel')
+        True
 
         >>> d.spellcheck_suggest('')
         []
@@ -543,8 +468,8 @@ class Hunspell:
         '''Return completions or corrections for the input phrase
 
         :param input_phrase: A string to find completions or corrections for
-        :type input_phrase: String
-        :rtype: A list of tuples of the form (<word>, <score>)
+
+        Returns a list of tuples of the form (<word>, <score>)
                 <score> can have these values:
                     0: This is a completion, i.e. input_phrase matches
                        the beginning of <word> (accent insensitive match)
@@ -590,8 +515,8 @@ class Hunspell:
         ('tenéis', 0)
 
         >>> h = Hunspell(['en_US'])
-        >>> h.suggest('camel')
-        [('camel', 0), ('camellia', 0), ('camelhair', 0), ('came', -1), ('Camel', -1), ('cameo', -1), ('came l', -1), ('camels', -1)]
+        >>> ('Camelot', 0) in h.suggest('camel') # Should work with aspell and hunspell
+        True
 
         >>> h = Hunspell(['fr_FR'])
         >>> h.suggest('differemmen')
@@ -602,9 +527,8 @@ class Hunspell:
         []
 
         >>> h = Hunspell(['None', 'en_US'])
-        >>> h.suggest('camel')
-        [('camel', 0), ('camellia', 0), ('camelhair', 0), ('came', -1), ('Camel', -1), ('cameo', -1), ('came l', -1), ('camels', -1)]
-
+        >>> ('Camelot', 0) in h.suggest('camel') # Should work with aspell and hunspell
+        True
         '''
         # pylint: enable=line-too-long
         if input_phrase in self._suggest_cache:
@@ -630,8 +554,6 @@ class Hunspell:
         # make sure input_phrase is in the internal normalization form (NFD):
         input_phrase = unicodedata.normalize(
             itb_util.NORMALIZATION_FORM_INTERNAL, input_phrase)
-        # But enchant and pyhunspell want NFC as input, make a copy in NFC:
-        input_phrase_nfc = unicodedata.normalize('NFC', input_phrase)
 
         suggested_words: Dict[str, int] = {}
         for dictionary in self._dictionaries:
@@ -639,21 +561,45 @@ class Hunspell:
                 if dictionary.word_pairs:
                     input_phrase_no_accents = itb_util.remove_accents(
                         input_phrase,
-                        keep=ACCENT_LANGUAGES[dictionary.language])
+                        keep=itb_util.ACCENT_LANGUAGES[dictionary.language])
                 # If the input phrase is longer than than the maximum
                 # word length in a dictionary, don’t try
                 # complete it, it just wastes time then.
                 if len(input_phrase) <= dictionary.max_word_len:
                     if dictionary.word_pairs:
-                        suggested_words.update([
-                            (x[0], 0)
-                            for x in dictionary.word_pairs
-                            if x[1].startswith(input_phrase_no_accents)])
+                        if IMPORT_REGEX_SUCCESFUL:
+                            regex_pattern = regex.compile(
+                                regex.escape(input_phrase_no_accents),
+                                regex.IGNORECASE)
+                            suggested_words.update([
+                                (x[0], 0)
+                                for x in dictionary.word_pairs
+                                if regex_pattern.match(x[1])])
+                        else:
+                            re_pattern = re.compile(
+                                re.escape(input_phrase_no_accents),
+                                re.IGNORECASE)
+                            suggested_words.update([
+                                (x[0], 0)
+                                for x in dictionary.word_pairs
+                                if re_pattern.match(x[1])])
                     else:
-                        suggested_words.update([
-                            (x, 0)
-                            for x in dictionary.words
-                            if x.startswith(input_phrase)])
+                        if IMPORT_REGEX_SUCCESFUL:
+                            regex_pattern = regex.compile(
+                                regex.escape(input_phrase),
+                                regex.IGNORECASE)
+                            suggested_words.update([
+                                (x, 0)
+                                for x in dictionary.words
+                                if regex_pattern.match(x)])
+                        else:
+                            re_pattern = re.compile(
+                                re.escape(input_phrase),
+                                re.IGNORECASE)
+                            suggested_words.update([
+                                (x, 0)
+                                for x in dictionary.words
+                                if re_pattern.match(x)])
                 if len(input_phrase) >= 4:
                     if dictionary.spellcheck(input_phrase):
                         # This is a valid word in this dictionary.
@@ -678,7 +624,8 @@ class Hunspell:
                                 and
                                 itb_util.remove_accents(
                                     suggestion,
-                                    keep=ACCENT_LANGUAGES[dictionary.language])
+                                    keep=itb_util.ACCENT_LANGUAGES[
+                                        dictionary.language])
                                 == input_phrase_no_accents):
                                 suggested_words[suggestion] = 0
                             else:
@@ -724,10 +671,10 @@ def main():
         stats.print_stats('hunspell', 25)
         stats.print_stats('enchant', 25)
 
-    if failed:
-        sys.exit(1)
-    else:
-        sys.exit(0)
+    LOGGER.info('itb_util.remove_accents() cache info: %s',
+                itb_util.remove_accents.cache_info())
+
+    sys.exit(failed)
 
 if __name__ == "__main__":
     main()
